# -*- coding: utf-8 -*-
"""
    pieces_test
    ~~~~~~~


    :copyright: © 2011 UfSoft.org - :email:`Pedro Algarvio (pedro@algarvio.me)`
    :license: BSD, see LICENSE for more details.
"""
import pygtk
pygtk.require('2.0')
import gtk

class PiecesBar(gtk.DrawingArea):
    # Draw in response to an expose-event
    __gsignals__ = { "expose-event": "override" }

    def __init__(self):
        gtk.DrawingArea.__init__(self)
        self.width = 0
        self.height = 0
        self.pieces = []
        self.colors = {}

        self.connect('size-allocate', self.on_size_allocate)
        self.connect('realize', self.on_realize)

        self.show()

    def on_realize(self, widget):
        map = widget.get_colormap()
        done_color = map.alloc_color("#4883b5", True, True)
        down_color = map.alloc_color("#fff600", True, True)
        wait_color = map.alloc_color("#3dff58", True, True)
        miss_color = map.alloc_color("#ff0000", True, True)
        outline_color = map.alloc_color("#888888")

        self.colors = {
            0: (miss_color.red/65535.0, miss_color.green/65535.0, miss_color.blue/65535.0),
            1: (wait_color.red/65535.0, wait_color.green/65535.0, wait_color.blue/65535.0),
            2: (down_color.red/65535.0, down_color.green/65535.0, down_color.blue/65535.0),
            3: (done_color.red/65535.0, done_color.green/65535.0, done_color.blue/65535.0),
        }
        self.gc_outline = widget.window.new_gc(foreground=outline_color)

    def on_size_allocate(self, widget, size):
        self.width = size.width
        self.height = size.height

    # Handle the expose-event by drawing
    def do_expose_event(self, event=None):

        num_pieces = len(self.pieces)

        if num_pieces < 1:
            self.clear()
            return None

        # Create the cairo context
        cr = self.window.cairo_create()
        # Restrict Cairo to the exposed area; avoid extra work
        if event:
            cr.rectangle(event.area.x, event.area.y,
                         event.area.width, event.area.height)
            cr.clip()

        # Fill the background
        cr.set_source_rgb(136, 136, 136)
        cr.rectangle(0, 0, self.width-1, self.height-1)
        cr.fill()

        width = self.width - 2
        pieces = self.collapse_pieces()

        start_pos = 1

        for state, wpercent in pieces:
            pwidth = width*wpercent
            cr.set_source_rgb(*self.colors[state])
            cr.rectangle(start_pos, 1, pwidth, self.height-2)
            cr.fill()
            start_pos += pwidth

    def collapse_pieces(self):
        num_pieces = len(self.pieces)*1.0
        opieces = self.pieces[:]
        npieces = []
        v = None
        n = None
        while True:
            if not opieces:
                break
            if v is None:
                v = opieces.pop(0)
                n = 1
            else:
                vv = opieces.pop(0)
                if v == vv:
                    n += 1
                else:
                    npieces.append((v, n/num_pieces))
                    v = vv
                    n = 1
        return npieces

    def draw_piece(self, piece, start_x, start_y, width, height):
        self.window.draw_rectangle(
            self.colors[piece], True, start_x, start_y, width, height
        )

    def clear(self):
        self.pieces = [0]
        self.update()

    def get_text(self):
        return ""

    def set_text(self, text):
        pass

pieces = PiecesBar()
pieces.pieces = [3, 2, 1, 1, 1, 0, 0, 2, 0, 1, 0, 2, 2, 2, 1, 2, 2, 0, 0, 0, 2,
                 1, 1, 1, 2, 0, 0, 2, 2, 1, 0, 2, 1, 0, 1, 1, 0, 3, 3, 3, 3, 3,
                 3, 3, 0, 3, 3, 0, 0, 2, 0, 2, 2, 2, 0, 2, 2, 1]*100

window = gtk.Window(gtk.WINDOW_TOPLEVEL)
window.set_default_size(500, 200)
def destroy(widget, data=None):
    gtk.main_quit()
window.connect("destroy", destroy)
vbox = gtk.VBox()
vbox.add(pieces)

hbox1 = gtk.HBox()
hbox1.pack_start(gtk.Label("Missing:"), False, False)
missing_color = gtk.ColorButton(window.get_colormap().alloc_color("#ff0000", True, True))
def on_missing_collor_changed(button):
    color = button.get_color()
    r, g, b = color.red, color.green, color.blue
    pieces.colors[0] = r/65535.0, g/65535.0, b/65535.0
    print 'Miss color RGB', pieces.colors[0]
    pieces.do_expose_event()
missing_color.connect("color-set", on_missing_collor_changed)
hbox1.pack_start(missing_color, False, False)
vbox.pack_start(hbox1, False, False)

hbox2 = gtk.HBox()
hbox2.pack_start(gtk.Label("Waiting:"), False, False)
waiting_color = gtk.ColorButton(window.get_colormap().alloc_color("#3dff58", True, True))
def on_waiting_collor_changed(button):
    color = button.get_color()
    r, g, b = color.red, color.green, color.blue
    pieces.colors[1] = r/65535.0, g/65535.0, b/65535.0
    print 'Wait color RGB', pieces.colors[1]
    pieces.do_expose_event()
waiting_color.connect("color-set", on_waiting_collor_changed)
hbox2.pack_start(waiting_color, False, False)
vbox.pack_start(hbox2, False, False)

hbox3 = gtk.HBox()
hbox3.pack_start(gtk.Label("Downloading:"), False, False)
down_color = gtk.ColorButton(window.get_colormap().alloc_color("#fff600", True, True))
def on_down_collor_changed(button):
    color = button.get_color()
    r, g, b = color.red, color.green, color.blue
    pieces.colors[2] = r/65535.0, g/65535.0, b/65535.0
    print 'Down color RGB', pieces.colors[2]
    pieces.do_expose_event()
down_color.connect("color-set", on_down_collor_changed)
hbox3.pack_start(down_color, False, False)
vbox.pack_start(hbox3, False, False)

hbox4 = gtk.HBox()
hbox4.pack_start(gtk.Label("Completed:"), False, False)
done_color = gtk.ColorButton(window.get_colormap().alloc_color("#4883b5", True, True))
def on_done_collor_changed(button):
    color = button.get_color()
    r, g, b = color.red, color.green, color.blue
    pieces.colors[3] = r/65535.0, g/65535.0, b/65535.0
    print 'Done color RGB', pieces.colors[3]
    pieces.do_expose_event()
done_color.connect("color-set", on_done_collor_changed)
hbox4.pack_start(done_color, False, False)
vbox.pack_start(hbox4, False, False)

window.add(vbox)
window.show_all()
gtk.main()

